/*******************************************************************************
* File Name: CommCounter.c  
* Version 3.0
*
*  Description:
*     The Counter component consists of a 8, 16, 24 or 32-bit counter with
*     a selectable period between 2 and 2^Width - 1.  
*
*   Note:
*     None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "CommCounter.h"

uint8 CommCounter_initVar = 0u;


/*******************************************************************************
* Function Name: CommCounter_Init
********************************************************************************
* Summary:
*     Initialize to the schematic state
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
*******************************************************************************/
void CommCounter_Init(void) 
{
        #if (!CommCounter_UsingFixedFunction && !CommCounter_ControlRegRemoved)
            uint8 ctrl;
        #endif /* (!CommCounter_UsingFixedFunction && !CommCounter_ControlRegRemoved) */
        
        #if(!CommCounter_UsingFixedFunction) 
            /* Interrupt State Backup for Critical Region*/
            uint8 CommCounter_interruptState;
        #endif /* (!CommCounter_UsingFixedFunction) */
        
        #if (CommCounter_UsingFixedFunction)
            /* Clear all bits but the enable bit (if it's already set for Timer operation */
            CommCounter_CONTROL &= CommCounter_CTRL_ENABLE;
            
            /* Clear the mode bits for continuous run mode */
            #if (CY_PSOC5A)
                CommCounter_CONTROL2 &= ((uint8)(~CommCounter_CTRL_MODE_MASK));
            #endif /* (CY_PSOC5A) */
            #if (CY_PSOC3 || CY_PSOC5LP)
                CommCounter_CONTROL3 &= ((uint8)(~CommCounter_CTRL_MODE_MASK));                
            #endif /* (CY_PSOC3 || CY_PSOC5LP) */
            /* Check if One Shot mode is enabled i.e. RunMode !=0*/
            #if (CommCounter_RunModeUsed != 0x0u)
                /* Set 3rd bit of Control register to enable one shot mode */
                CommCounter_CONTROL |= CommCounter_ONESHOT;
            #endif /* (CommCounter_RunModeUsed != 0x0u) */
            
            /* Set the IRQ to use the status register interrupts */
            CommCounter_CONTROL2 |= CommCounter_CTRL2_IRQ_SEL;
            
            /* Clear and Set SYNCTC and SYNCCMP bits of RT1 register */
            CommCounter_RT1 &= ((uint8)(~CommCounter_RT1_MASK));
            CommCounter_RT1 |= CommCounter_SYNC;     
                    
            /*Enable DSI Sync all all inputs of the Timer*/
            CommCounter_RT1 &= ((uint8)(~CommCounter_SYNCDSI_MASK));
            CommCounter_RT1 |= CommCounter_SYNCDSI_EN;

        #else
            #if(!CommCounter_ControlRegRemoved)
            /* Set the default compare mode defined in the parameter */
            ctrl = CommCounter_CONTROL & ((uint8)(~CommCounter_CTRL_CMPMODE_MASK));
            CommCounter_CONTROL = ctrl | CommCounter_DEFAULT_COMPARE_MODE;
            
            /* Set the default capture mode defined in the parameter */
            ctrl = CommCounter_CONTROL & ((uint8)(~CommCounter_CTRL_CAPMODE_MASK));
            
            #if( 0 != CommCounter_CAPTURE_MODE_CONF)
                CommCounter_CONTROL = ctrl | CommCounter_DEFAULT_CAPTURE_MODE;
            #else
                CommCounter_CONTROL = ctrl;
            #endif /* 0 != CommCounter_CAPTURE_MODE */ 
            
            #endif /* (!CommCounter_ControlRegRemoved) */
        #endif /* (CommCounter_UsingFixedFunction) */
        
        /* Clear all data in the FIFO's */
        #if (!CommCounter_UsingFixedFunction)
            CommCounter_ClearFIFO();
        #endif /* (!CommCounter_UsingFixedFunction) */
        
        /* Set Initial values from Configuration */
        CommCounter_WritePeriod(CommCounter_INIT_PERIOD_VALUE);
        #if (!(CommCounter_UsingFixedFunction && (CY_PSOC5A)))
            CommCounter_WriteCounter(CommCounter_INIT_COUNTER_VALUE);
        #endif /* (!(CommCounter_UsingFixedFunction && (CY_PSOC5A))) */
        CommCounter_SetInterruptMode(CommCounter_INIT_INTERRUPTS_MASK);
        
        #if (!CommCounter_UsingFixedFunction)
            /* Read the status register to clear the unwanted interrupts */
            (void)CommCounter_ReadStatusRegister();
            /* Set the compare value (only available to non-fixed function implementation */
            CommCounter_WriteCompare(CommCounter_INIT_COMPARE_VALUE);
            /* Use the interrupt output of the status register for IRQ output */
            
            /* CyEnterCriticalRegion and CyExitCriticalRegion are used to mark following region critical*/
            /* Enter Critical Region*/
            CommCounter_interruptState = CyEnterCriticalSection();
            
            CommCounter_STATUS_AUX_CTRL |= CommCounter_STATUS_ACTL_INT_EN_MASK;
            
            /* Exit Critical Region*/
            CyExitCriticalSection(CommCounter_interruptState);
            
        #endif /* (!CommCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: CommCounter_Enable
********************************************************************************
* Summary:
*     Enable the Counter
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: 
*   If the Enable mode is set to Hardware only then this function has no effect 
*   on the operation of the counter.
*
*******************************************************************************/
void CommCounter_Enable(void) 
{
    /* Globally Enable the Fixed Function Block chosen */
    #if (CommCounter_UsingFixedFunction)
        CommCounter_GLOBAL_ENABLE |= CommCounter_BLOCK_EN_MASK;
        CommCounter_GLOBAL_STBY_ENABLE |= CommCounter_BLOCK_STBY_EN_MASK;
    #endif /* (CommCounter_UsingFixedFunction) */  
        
    /* Enable the counter from the control register  */
    /* If Fixed Function then make sure Mode is set correctly */
    /* else make sure reset is clear */
    #if(!CommCounter_ControlRegRemoved || CommCounter_UsingFixedFunction)
        CommCounter_CONTROL |= CommCounter_CTRL_ENABLE;                
    #endif /* (!CommCounter_ControlRegRemoved || CommCounter_UsingFixedFunction) */
    
}


/*******************************************************************************
* Function Name: CommCounter_Start
********************************************************************************
* Summary:
*  Enables the counter for operation 
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  CommCounter_initVar: Is modified when this function is called for the  
*   first time. Is used to ensure that initialization happens only once.
*
*******************************************************************************/
void CommCounter_Start(void) 
{
    if(CommCounter_initVar == 0u)
    {
        CommCounter_Init();
        
        CommCounter_initVar = 1u; /* Clear this bit for Initialization */        
    }
    
    /* Enable the Counter */
    CommCounter_Enable();        
}


/*******************************************************************************
* Function Name: CommCounter_Stop
********************************************************************************
* Summary:
* Halts the counter, but does not change any modes or disable interrupts.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: If the Enable mode is set to Hardware only then this function
*               has no effect on the operation of the counter.
*
*******************************************************************************/
void CommCounter_Stop(void) 
{
    /* Disable Counter */
    #if(!CommCounter_ControlRegRemoved || CommCounter_UsingFixedFunction)
        CommCounter_CONTROL &= ((uint8)(~CommCounter_CTRL_ENABLE));        
    #endif /* (!CommCounter_ControlRegRemoved || CommCounter_UsingFixedFunction) */
    
    /* Globally disable the Fixed Function Block chosen */
    #if (CommCounter_UsingFixedFunction)
        CommCounter_GLOBAL_ENABLE &= ((uint8)(~CommCounter_BLOCK_EN_MASK));
        CommCounter_GLOBAL_STBY_ENABLE &= ((uint8)(~CommCounter_BLOCK_STBY_EN_MASK));
    #endif /* (CommCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: CommCounter_SetInterruptMode
********************************************************************************
* Summary:
* Configures which interrupt sources are enabled to generate the final interrupt
*
* Parameters:  
*  InterruptsMask: This parameter is an or'd collection of the status bits
*                   which will be allowed to generate the counters interrupt.   
*
* Return: 
*  void
*
*******************************************************************************/
void CommCounter_SetInterruptMode(uint8 interruptsMask) 
{
    CommCounter_STATUS_MASK = interruptsMask;
}


/*******************************************************************************
* Function Name: CommCounter_ReadStatusRegister
********************************************************************************
* Summary:
*   Reads the status register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the status register
*
* Side Effects:
*   Status register bits may be clear on read. 
*
*******************************************************************************/
uint8   CommCounter_ReadStatusRegister(void) 
{
    return CommCounter_STATUS;
}


#if(!CommCounter_ControlRegRemoved)
/*******************************************************************************
* Function Name: CommCounter_ReadControlRegister
********************************************************************************
* Summary:
*   Reads the control register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
uint8   CommCounter_ReadControlRegister(void) 
{
    return CommCounter_CONTROL;
}


/*******************************************************************************
* Function Name: CommCounter_WriteControlRegister
********************************************************************************
* Summary:
*   Sets the bit-field of the control register.  This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
void    CommCounter_WriteControlRegister(uint8 control) 
{
    CommCounter_CONTROL = control;
}

#endif  /* (!CommCounter_ControlRegRemoved) */


#if (!(CommCounter_UsingFixedFunction && (CY_PSOC5A)))
/*******************************************************************************
* Function Name: CommCounter_WriteCounter
********************************************************************************
* Summary:
*   This funtion is used to set the counter to a specific value
*
* Parameters:  
*  counter:  New counter value. 
*
* Return: 
*  void 
*
*******************************************************************************/
void CommCounter_WriteCounter(uint8 counter) \
                                   
{
    #if(CommCounter_UsingFixedFunction)
        /* assert if block is already enabled */
        CYASSERT (0u == (CommCounter_GLOBAL_ENABLE & CommCounter_BLOCK_EN_MASK));
        /* If block is disabled, enable it and then write the counter */
        CommCounter_GLOBAL_ENABLE |= CommCounter_BLOCK_EN_MASK;
        CY_SET_REG16(CommCounter_COUNTER_LSB_PTR, (uint16)counter);
        CommCounter_GLOBAL_ENABLE &= ((uint8)(~CommCounter_BLOCK_EN_MASK));
    #else
        CY_SET_REG8(CommCounter_COUNTER_LSB_PTR, counter);
    #endif /* (CommCounter_UsingFixedFunction) */
}
#endif /* (!(CommCounter_UsingFixedFunction && (CY_PSOC5A))) */


/*******************************************************************************
* Function Name: CommCounter_ReadCounter
********************************************************************************
* Summary:
* Returns the current value of the counter.  It doesn't matter
* if the counter is enabled or running.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint8) The present value of the counter.
*
*******************************************************************************/
uint8 CommCounter_ReadCounter(void) 
{
    /* Force capture by reading Accumulator */
    /* Must first do a software capture to be able to read the counter */
    /* It is up to the user code to make sure there isn't already captured data in the FIFO */
    #if(CommCounter_UsingFixedFunction)
		(void)CY_GET_REG16(CommCounter_COUNTER_LSB_PTR);
	#else
		(void)CY_GET_REG8(CommCounter_COUNTER_LSB_PTR_8BIT);
	#endif/* (CommCounter_UsingFixedFunction) */
    
    /* Read the data from the FIFO (or capture register for Fixed Function)*/
    #if(CommCounter_UsingFixedFunction)
        return ((uint8)CY_GET_REG16(CommCounter_STATICCOUNT_LSB_PTR));
    #else
        return (CY_GET_REG8(CommCounter_STATICCOUNT_LSB_PTR));
    #endif /* (CommCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: CommCounter_ReadCapture
********************************************************************************
* Summary:
*   This function returns the last value captured.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) Present Capture value.
*
*******************************************************************************/
uint8 CommCounter_ReadCapture(void) 
{
    #if(CommCounter_UsingFixedFunction)
        return ((uint8)CY_GET_REG16(CommCounter_STATICCOUNT_LSB_PTR));
    #else
        return (CY_GET_REG8(CommCounter_STATICCOUNT_LSB_PTR));
    #endif /* (CommCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: CommCounter_WritePeriod
********************************************************************************
* Summary:
* Changes the period of the counter.  The new period 
* will be loaded the next time terminal count is detected.
*
* Parameters:  
*  period: (uint8) A value of 0 will result in
*         the counter remaining at zero.  
*
* Return: 
*  void
*
*******************************************************************************/
void CommCounter_WritePeriod(uint8 period) 
{
    #if(CommCounter_UsingFixedFunction)
        CY_SET_REG16(CommCounter_PERIOD_LSB_PTR,(uint16)period);
    #else
        CY_SET_REG8(CommCounter_PERIOD_LSB_PTR, period);
    #endif /* (CommCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: CommCounter_ReadPeriod
********************************************************************************
* Summary:
* Reads the current period value without affecting counter operation.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint8) Present period value.
*
*******************************************************************************/
uint8 CommCounter_ReadPeriod(void) 
{
    #if(CommCounter_UsingFixedFunction)
        return ((uint8)CY_GET_REG16(CommCounter_PERIOD_LSB_PTR));
    #else
        return (CY_GET_REG8(CommCounter_PERIOD_LSB_PTR));
    #endif /* (CommCounter_UsingFixedFunction) */
}


#if (!CommCounter_UsingFixedFunction)
/*******************************************************************************
* Function Name: CommCounter_WriteCompare
********************************************************************************
* Summary:
* Changes the compare value.  The compare output will 
* reflect the new value on the next UDB clock.  The compare output will be 
* driven high when the present counter value compares true based on the 
* configured compare mode setting. 
*
* Parameters:  
*  Compare:  New compare value. 
*
* Return: 
*  void
*
*******************************************************************************/
void CommCounter_WriteCompare(uint8 compare) \
                                   
{
    #if(CommCounter_UsingFixedFunction)
        CY_SET_REG16(CommCounter_COMPARE_LSB_PTR, (uint16)compare);
    #else
        CY_SET_REG8(CommCounter_COMPARE_LSB_PTR, compare);
    #endif /* (CommCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: CommCounter_ReadCompare
********************************************************************************
* Summary:
* Returns the compare value.
*
* Parameters:  
*  void:
*
* Return: 
*  (uint8) Present compare value.
*
*******************************************************************************/
uint8 CommCounter_ReadCompare(void) 
{
    return (CY_GET_REG8(CommCounter_COMPARE_LSB_PTR));
}


#if (CommCounter_COMPARE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: CommCounter_SetCompareMode
********************************************************************************
* Summary:
*  Sets the software controlled Compare Mode.
*
* Parameters:
*  compareMode:  Compare Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void CommCounter_SetCompareMode(uint8 compareMode) 
{
    /* Clear the compare mode bits in the control register */
    CommCounter_CONTROL &= ((uint8)(~CommCounter_CTRL_CMPMODE_MASK));
    
    /* Write the new setting */
    CommCounter_CONTROL |= compareMode;
}
#endif  /* (CommCounter_COMPARE_MODE_SOFTWARE) */


#if (CommCounter_CAPTURE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: CommCounter_SetCaptureMode
********************************************************************************
* Summary:
*  Sets the software controlled Capture Mode.
*
* Parameters:
*  captureMode:  Capture Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void CommCounter_SetCaptureMode(uint8 captureMode) 
{
    /* Clear the capture mode bits in the control register */
    CommCounter_CONTROL &= ((uint8)(~CommCounter_CTRL_CAPMODE_MASK));
    
    /* Write the new setting */
    CommCounter_CONTROL |= ((uint8)((uint8)captureMode << CommCounter_CTRL_CAPMODE0_SHIFT));
}
#endif  /* (CommCounter_CAPTURE_MODE_SOFTWARE) */


/*******************************************************************************
* Function Name: CommCounter_ClearFIFO
********************************************************************************
* Summary:
*   This function clears all capture data from the capture FIFO
*
* Parameters:  
*  void:
*
* Return: 
*  None
*
*******************************************************************************/
void CommCounter_ClearFIFO(void) 
{

    while(0u != (CommCounter_ReadStatusRegister() & CommCounter_STATUS_FIFONEMP))
    {
        (void)CommCounter_ReadCapture();
    }

}
#endif  /* (!CommCounter_UsingFixedFunction) */


/* [] END OF FILE */

