/*******************************************************************************
* File Name: CommCounter_PM.c  
* Version 3.0
*
*  Description:
*    This file provides the power management source code to API for the
*    Counter.  
*
*   Note:
*     None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "CommCounter.h"

static CommCounter_backupStruct CommCounter_backup;


/*******************************************************************************
* Function Name: CommCounter_SaveConfig
********************************************************************************
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  CommCounter_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
*******************************************************************************/
void CommCounter_SaveConfig(void) 
{
    #if (!CommCounter_UsingFixedFunction)

        CommCounter_backup.CounterUdb = CommCounter_ReadCounter();

        #if(!CommCounter_ControlRegRemoved)
            CommCounter_backup.CounterControlRegister = CommCounter_ReadControlRegister();
        #endif /* (!CommCounter_ControlRegRemoved) */

    #endif /* (!CommCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: CommCounter_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  CommCounter_backup:  Variables of this global structure are used to 
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void CommCounter_RestoreConfig(void) 
{      
    #if (!CommCounter_UsingFixedFunction)

       CommCounter_WriteCounter(CommCounter_backup.CounterUdb);

        #if(!CommCounter_ControlRegRemoved)
            CommCounter_WriteControlRegister(CommCounter_backup.CounterControlRegister);
        #endif /* (!CommCounter_ControlRegRemoved) */

    #endif /* (!CommCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: CommCounter_Sleep
********************************************************************************
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  CommCounter_backup.enableState:  Is modified depending on the enable 
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void CommCounter_Sleep(void) 
{
    #if(!CommCounter_ControlRegRemoved)
        /* Save Counter's enable state */
        if(CommCounter_CTRL_ENABLE == (CommCounter_CONTROL & CommCounter_CTRL_ENABLE))
        {
            /* Counter is enabled */
            CommCounter_backup.CounterEnableState = 1u;
        }
        else
        {
            /* Counter is disabled */
            CommCounter_backup.CounterEnableState = 0u;
        }
    #else
        CommCounter_backup.CounterEnableState = 1u;
        if(CommCounter_backup.CounterEnableState != 0u)
        {
            CommCounter_backup.CounterEnableState = 0u;
        }
    #endif /* (!CommCounter_ControlRegRemoved) */
    
    CommCounter_Stop();
    CommCounter_SaveConfig();
}


/*******************************************************************************
* Function Name: CommCounter_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  CommCounter_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void CommCounter_Wakeup(void) 
{
    CommCounter_RestoreConfig();
    #if(!CommCounter_ControlRegRemoved)
        if(CommCounter_backup.CounterEnableState == 1u)
        {
            /* Enable Counter's operation */
            CommCounter_Enable();
        } /* Do nothing if Counter was disabled before */    
    #endif /* (!CommCounter_ControlRegRemoved) */
    
}


/* [] END OF FILE */
