/*******************************************************************************
* File Name: CommTimer_PM.c
* Version 2.70
*
*  Description:
*     This file provides the power management source code to API for the
*     Timer.
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "CommTimer.h"

static CommTimer_backupStruct CommTimer_backup;


/*******************************************************************************
* Function Name: CommTimer_SaveConfig
********************************************************************************
*
* Summary:
*     Save the current user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  CommTimer_backup:  Variables of this global structure are modified to
*  store the values of non retention configuration registers when Sleep() API is
*  called.
*
*******************************************************************************/
void CommTimer_SaveConfig(void) 
{
    #if (!CommTimer_UsingFixedFunction)
        CommTimer_backup.TimerUdb = CommTimer_ReadCounter();
        CommTimer_backup.InterruptMaskValue = CommTimer_STATUS_MASK;
        #if (CommTimer_UsingHWCaptureCounter)
            CommTimer_backup.TimerCaptureCounter = CommTimer_ReadCaptureCount();
        #endif /* Back Up capture counter register  */

        #if(!CommTimer_UDB_CONTROL_REG_REMOVED)
            CommTimer_backup.TimerControlRegister = CommTimer_ReadControlRegister();
        #endif /* Backup the enable state of the Timer component */
    #endif /* Backup non retention registers in UDB implementation. All fixed function registers are retention */
}


/*******************************************************************************
* Function Name: CommTimer_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  CommTimer_backup:  Variables of this global structure are used to
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void CommTimer_RestoreConfig(void) 
{   
    #if (!CommTimer_UsingFixedFunction)

        CommTimer_WriteCounter(CommTimer_backup.TimerUdb);
        CommTimer_STATUS_MASK =CommTimer_backup.InterruptMaskValue;
        #if (CommTimer_UsingHWCaptureCounter)
            CommTimer_SetCaptureCount(CommTimer_backup.TimerCaptureCounter);
        #endif /* Restore Capture counter register*/

        #if(!CommTimer_UDB_CONTROL_REG_REMOVED)
            CommTimer_WriteControlRegister(CommTimer_backup.TimerControlRegister);
        #endif /* Restore the enable state of the Timer component */
    #endif /* Restore non retention registers in the UDB implementation only */
}


/*******************************************************************************
* Function Name: CommTimer_Sleep
********************************************************************************
*
* Summary:
*     Stop and Save the user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  CommTimer_backup.TimerEnableState:  Is modified depending on the
*  enable state of the block before entering sleep mode.
*
*******************************************************************************/
void CommTimer_Sleep(void) 
{
    #if(!CommTimer_UDB_CONTROL_REG_REMOVED)
        /* Save Counter's enable state */
        if(CommTimer_CTRL_ENABLE == (CommTimer_CONTROL & CommTimer_CTRL_ENABLE))
        {
            /* Timer is enabled */
            CommTimer_backup.TimerEnableState = 1u;
        }
        else
        {
            /* Timer is disabled */
            CommTimer_backup.TimerEnableState = 0u;
        }
    #endif /* Back up enable state from the Timer control register */
    CommTimer_Stop();
    CommTimer_SaveConfig();
}


/*******************************************************************************
* Function Name: CommTimer_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  CommTimer_backup.enableState:  Is used to restore the enable state of
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void CommTimer_Wakeup(void) 
{
    CommTimer_RestoreConfig();
    #if(!CommTimer_UDB_CONTROL_REG_REMOVED)
        if(CommTimer_backup.TimerEnableState == 1u)
        {     /* Enable Timer's operation */
                CommTimer_Enable();
        } /* Do nothing if Timer was disabled before */
    #endif /* Remove this code section if Control register is removed */
}


/* [] END OF FILE */
