/*******************************************************************************
* File Name: PWM_DISPLAY_PM.c
* Version 3.30
*
* Description:
*  This file provides the power management source code to API for the
*  PWM.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "PWM_DISPLAY.h"

static PWM_DISPLAY_backupStruct PWM_DISPLAY_backup;


/*******************************************************************************
* Function Name: PWM_DISPLAY_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the current user configuration of the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  PWM_DISPLAY_backup:  Variables of this global structure are modified to
*  store the values of non retention configuration registers when Sleep() API is
*  called.
*
*******************************************************************************/
void PWM_DISPLAY_SaveConfig(void) 
{

    #if(!PWM_DISPLAY_UsingFixedFunction)
        #if(!PWM_DISPLAY_PWMModeIsCenterAligned)
            PWM_DISPLAY_backup.PWMPeriod = PWM_DISPLAY_ReadPeriod();
        #endif /* (!PWM_DISPLAY_PWMModeIsCenterAligned) */
        PWM_DISPLAY_backup.PWMUdb = PWM_DISPLAY_ReadCounter();
        #if (PWM_DISPLAY_UseStatus)
            PWM_DISPLAY_backup.InterruptMaskValue = PWM_DISPLAY_STATUS_MASK;
        #endif /* (PWM_DISPLAY_UseStatus) */

        #if(PWM_DISPLAY_DeadBandMode == PWM_DISPLAY__B_PWM__DBM_256_CLOCKS || \
            PWM_DISPLAY_DeadBandMode == PWM_DISPLAY__B_PWM__DBM_2_4_CLOCKS)
            PWM_DISPLAY_backup.PWMdeadBandValue = PWM_DISPLAY_ReadDeadTime();
        #endif /*  deadband count is either 2-4 clocks or 256 clocks */

        #if(PWM_DISPLAY_KillModeMinTime)
             PWM_DISPLAY_backup.PWMKillCounterPeriod = PWM_DISPLAY_ReadKillTime();
        #endif /* (PWM_DISPLAY_KillModeMinTime) */

        #if(PWM_DISPLAY_UseControl)
            PWM_DISPLAY_backup.PWMControlRegister = PWM_DISPLAY_ReadControlRegister();
        #endif /* (PWM_DISPLAY_UseControl) */
    #endif  /* (!PWM_DISPLAY_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: PWM_DISPLAY_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration of the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  PWM_DISPLAY_backup:  Variables of this global structure are used to
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void PWM_DISPLAY_RestoreConfig(void) 
{
        #if(!PWM_DISPLAY_UsingFixedFunction)
            #if(!PWM_DISPLAY_PWMModeIsCenterAligned)
                PWM_DISPLAY_WritePeriod(PWM_DISPLAY_backup.PWMPeriod);
            #endif /* (!PWM_DISPLAY_PWMModeIsCenterAligned) */

            PWM_DISPLAY_WriteCounter(PWM_DISPLAY_backup.PWMUdb);

            #if (PWM_DISPLAY_UseStatus)
                PWM_DISPLAY_STATUS_MASK = PWM_DISPLAY_backup.InterruptMaskValue;
            #endif /* (PWM_DISPLAY_UseStatus) */

            #if(PWM_DISPLAY_DeadBandMode == PWM_DISPLAY__B_PWM__DBM_256_CLOCKS || \
                PWM_DISPLAY_DeadBandMode == PWM_DISPLAY__B_PWM__DBM_2_4_CLOCKS)
                PWM_DISPLAY_WriteDeadTime(PWM_DISPLAY_backup.PWMdeadBandValue);
            #endif /* deadband count is either 2-4 clocks or 256 clocks */

            #if(PWM_DISPLAY_KillModeMinTime)
                PWM_DISPLAY_WriteKillTime(PWM_DISPLAY_backup.PWMKillCounterPeriod);
            #endif /* (PWM_DISPLAY_KillModeMinTime) */

            #if(PWM_DISPLAY_UseControl)
                PWM_DISPLAY_WriteControlRegister(PWM_DISPLAY_backup.PWMControlRegister);
            #endif /* (PWM_DISPLAY_UseControl) */
        #endif  /* (!PWM_DISPLAY_UsingFixedFunction) */
    }


/*******************************************************************************
* Function Name: PWM_DISPLAY_Sleep
********************************************************************************
*
* Summary:
*  Disables block's operation and saves the user configuration. Should be called
*  just prior to entering sleep.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  PWM_DISPLAY_backup.PWMEnableState:  Is modified depending on the enable
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void PWM_DISPLAY_Sleep(void) 
{
    #if(PWM_DISPLAY_UseControl)
        if(PWM_DISPLAY_CTRL_ENABLE == (PWM_DISPLAY_CONTROL & PWM_DISPLAY_CTRL_ENABLE))
        {
            /*Component is enabled */
            PWM_DISPLAY_backup.PWMEnableState = 1u;
        }
        else
        {
            /* Component is disabled */
            PWM_DISPLAY_backup.PWMEnableState = 0u;
        }
    #endif /* (PWM_DISPLAY_UseControl) */

    /* Stop component */
    PWM_DISPLAY_Stop();

    /* Save registers configuration */
    PWM_DISPLAY_SaveConfig();
}


/*******************************************************************************
* Function Name: PWM_DISPLAY_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration. Should be called just after
*  awaking from sleep.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  PWM_DISPLAY_backup.pwmEnable:  Is used to restore the enable state of
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void PWM_DISPLAY_Wakeup(void) 
{
     /* Restore registers values */
    PWM_DISPLAY_RestoreConfig();

    if(PWM_DISPLAY_backup.PWMEnableState != 0u)
    {
        /* Enable component's operation */
        PWM_DISPLAY_Enable();
    } /* Do nothing if component's block was disabled before */

}


/* [] END OF FILE */
