/*******************************************************************************
* File Name: PWM_DMA_PM.c
* Version 3.30
*
* Description:
*  This file provides the power management source code to API for the
*  PWM.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "PWM_DMA.h"

static PWM_DMA_backupStruct PWM_DMA_backup;


/*******************************************************************************
* Function Name: PWM_DMA_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the current user configuration of the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  PWM_DMA_backup:  Variables of this global structure are modified to
*  store the values of non retention configuration registers when Sleep() API is
*  called.
*
*******************************************************************************/
void PWM_DMA_SaveConfig(void) 
{

    #if(!PWM_DMA_UsingFixedFunction)
        #if(!PWM_DMA_PWMModeIsCenterAligned)
            PWM_DMA_backup.PWMPeriod = PWM_DMA_ReadPeriod();
        #endif /* (!PWM_DMA_PWMModeIsCenterAligned) */
        PWM_DMA_backup.PWMUdb = PWM_DMA_ReadCounter();
        #if (PWM_DMA_UseStatus)
            PWM_DMA_backup.InterruptMaskValue = PWM_DMA_STATUS_MASK;
        #endif /* (PWM_DMA_UseStatus) */

        #if(PWM_DMA_DeadBandMode == PWM_DMA__B_PWM__DBM_256_CLOCKS || \
            PWM_DMA_DeadBandMode == PWM_DMA__B_PWM__DBM_2_4_CLOCKS)
            PWM_DMA_backup.PWMdeadBandValue = PWM_DMA_ReadDeadTime();
        #endif /*  deadband count is either 2-4 clocks or 256 clocks */

        #if(PWM_DMA_KillModeMinTime)
             PWM_DMA_backup.PWMKillCounterPeriod = PWM_DMA_ReadKillTime();
        #endif /* (PWM_DMA_KillModeMinTime) */

        #if(PWM_DMA_UseControl)
            PWM_DMA_backup.PWMControlRegister = PWM_DMA_ReadControlRegister();
        #endif /* (PWM_DMA_UseControl) */
    #endif  /* (!PWM_DMA_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: PWM_DMA_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration of the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  PWM_DMA_backup:  Variables of this global structure are used to
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void PWM_DMA_RestoreConfig(void) 
{
        #if(!PWM_DMA_UsingFixedFunction)
            #if(!PWM_DMA_PWMModeIsCenterAligned)
                PWM_DMA_WritePeriod(PWM_DMA_backup.PWMPeriod);
            #endif /* (!PWM_DMA_PWMModeIsCenterAligned) */

            PWM_DMA_WriteCounter(PWM_DMA_backup.PWMUdb);

            #if (PWM_DMA_UseStatus)
                PWM_DMA_STATUS_MASK = PWM_DMA_backup.InterruptMaskValue;
            #endif /* (PWM_DMA_UseStatus) */

            #if(PWM_DMA_DeadBandMode == PWM_DMA__B_PWM__DBM_256_CLOCKS || \
                PWM_DMA_DeadBandMode == PWM_DMA__B_PWM__DBM_2_4_CLOCKS)
                PWM_DMA_WriteDeadTime(PWM_DMA_backup.PWMdeadBandValue);
            #endif /* deadband count is either 2-4 clocks or 256 clocks */

            #if(PWM_DMA_KillModeMinTime)
                PWM_DMA_WriteKillTime(PWM_DMA_backup.PWMKillCounterPeriod);
            #endif /* (PWM_DMA_KillModeMinTime) */

            #if(PWM_DMA_UseControl)
                PWM_DMA_WriteControlRegister(PWM_DMA_backup.PWMControlRegister);
            #endif /* (PWM_DMA_UseControl) */
        #endif  /* (!PWM_DMA_UsingFixedFunction) */
    }


/*******************************************************************************
* Function Name: PWM_DMA_Sleep
********************************************************************************
*
* Summary:
*  Disables block's operation and saves the user configuration. Should be called
*  just prior to entering sleep.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  PWM_DMA_backup.PWMEnableState:  Is modified depending on the enable
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void PWM_DMA_Sleep(void) 
{
    #if(PWM_DMA_UseControl)
        if(PWM_DMA_CTRL_ENABLE == (PWM_DMA_CONTROL & PWM_DMA_CTRL_ENABLE))
        {
            /*Component is enabled */
            PWM_DMA_backup.PWMEnableState = 1u;
        }
        else
        {
            /* Component is disabled */
            PWM_DMA_backup.PWMEnableState = 0u;
        }
    #endif /* (PWM_DMA_UseControl) */

    /* Stop component */
    PWM_DMA_Stop();

    /* Save registers configuration */
    PWM_DMA_SaveConfig();
}


/*******************************************************************************
* Function Name: PWM_DMA_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration. Should be called just after
*  awaking from sleep.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  PWM_DMA_backup.pwmEnable:  Is used to restore the enable state of
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void PWM_DMA_Wakeup(void) 
{
     /* Restore registers values */
    PWM_DMA_RestoreConfig();

    if(PWM_DMA_backup.PWMEnableState != 0u)
    {
        /* Enable component's operation */
        PWM_DMA_Enable();
    } /* Do nothing if component's block was disabled before */

}


/* [] END OF FILE */
