/*******************************************************************************
* File Name: PosPulseCounter_PM.c  
* Version 3.0
*
*  Description:
*    This file provides the power management source code to API for the
*    Counter.  
*
*   Note:
*     None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "PosPulseCounter.h"

static PosPulseCounter_backupStruct PosPulseCounter_backup;


/*******************************************************************************
* Function Name: PosPulseCounter_SaveConfig
********************************************************************************
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  PosPulseCounter_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
*******************************************************************************/
void PosPulseCounter_SaveConfig(void) 
{
    #if (!PosPulseCounter_UsingFixedFunction)

        PosPulseCounter_backup.CounterUdb = PosPulseCounter_ReadCounter();

        #if(!PosPulseCounter_ControlRegRemoved)
            PosPulseCounter_backup.CounterControlRegister = PosPulseCounter_ReadControlRegister();
        #endif /* (!PosPulseCounter_ControlRegRemoved) */

    #endif /* (!PosPulseCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: PosPulseCounter_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  PosPulseCounter_backup:  Variables of this global structure are used to 
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void PosPulseCounter_RestoreConfig(void) 
{      
    #if (!PosPulseCounter_UsingFixedFunction)

       PosPulseCounter_WriteCounter(PosPulseCounter_backup.CounterUdb);

        #if(!PosPulseCounter_ControlRegRemoved)
            PosPulseCounter_WriteControlRegister(PosPulseCounter_backup.CounterControlRegister);
        #endif /* (!PosPulseCounter_ControlRegRemoved) */

    #endif /* (!PosPulseCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: PosPulseCounter_Sleep
********************************************************************************
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  PosPulseCounter_backup.enableState:  Is modified depending on the enable 
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void PosPulseCounter_Sleep(void) 
{
    #if(!PosPulseCounter_ControlRegRemoved)
        /* Save Counter's enable state */
        if(PosPulseCounter_CTRL_ENABLE == (PosPulseCounter_CONTROL & PosPulseCounter_CTRL_ENABLE))
        {
            /* Counter is enabled */
            PosPulseCounter_backup.CounterEnableState = 1u;
        }
        else
        {
            /* Counter is disabled */
            PosPulseCounter_backup.CounterEnableState = 0u;
        }
    #else
        PosPulseCounter_backup.CounterEnableState = 1u;
        if(PosPulseCounter_backup.CounterEnableState != 0u)
        {
            PosPulseCounter_backup.CounterEnableState = 0u;
        }
    #endif /* (!PosPulseCounter_ControlRegRemoved) */
    
    PosPulseCounter_Stop();
    PosPulseCounter_SaveConfig();
}


/*******************************************************************************
* Function Name: PosPulseCounter_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  PosPulseCounter_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void PosPulseCounter_Wakeup(void) 
{
    PosPulseCounter_RestoreConfig();
    #if(!PosPulseCounter_ControlRegRemoved)
        if(PosPulseCounter_backup.CounterEnableState == 1u)
        {
            /* Enable Counter's operation */
            PosPulseCounter_Enable();
        } /* Do nothing if Counter was disabled before */    
    #endif /* (!PosPulseCounter_ControlRegRemoved) */
    
}


/* [] END OF FILE */
