/*******************************************************************************
* File Name: SingleWireCommCounter.c  
* Version 3.0
*
*  Description:
*     The Counter component consists of a 8, 16, 24 or 32-bit counter with
*     a selectable period between 2 and 2^Width - 1.  
*
*   Note:
*     None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "SingleWireCommCounter.h"

uint8 SingleWireCommCounter_initVar = 0u;


/*******************************************************************************
* Function Name: SingleWireCommCounter_Init
********************************************************************************
* Summary:
*     Initialize to the schematic state
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
*******************************************************************************/
void SingleWireCommCounter_Init(void) 
{
        #if (!SingleWireCommCounter_UsingFixedFunction && !SingleWireCommCounter_ControlRegRemoved)
            uint8 ctrl;
        #endif /* (!SingleWireCommCounter_UsingFixedFunction && !SingleWireCommCounter_ControlRegRemoved) */
        
        #if(!SingleWireCommCounter_UsingFixedFunction) 
            /* Interrupt State Backup for Critical Region*/
            uint8 SingleWireCommCounter_interruptState;
        #endif /* (!SingleWireCommCounter_UsingFixedFunction) */
        
        #if (SingleWireCommCounter_UsingFixedFunction)
            /* Clear all bits but the enable bit (if it's already set for Timer operation */
            SingleWireCommCounter_CONTROL &= SingleWireCommCounter_CTRL_ENABLE;
            
            /* Clear the mode bits for continuous run mode */
            #if (CY_PSOC5A)
                SingleWireCommCounter_CONTROL2 &= ((uint8)(~SingleWireCommCounter_CTRL_MODE_MASK));
            #endif /* (CY_PSOC5A) */
            #if (CY_PSOC3 || CY_PSOC5LP)
                SingleWireCommCounter_CONTROL3 &= ((uint8)(~SingleWireCommCounter_CTRL_MODE_MASK));                
            #endif /* (CY_PSOC3 || CY_PSOC5LP) */
            /* Check if One Shot mode is enabled i.e. RunMode !=0*/
            #if (SingleWireCommCounter_RunModeUsed != 0x0u)
                /* Set 3rd bit of Control register to enable one shot mode */
                SingleWireCommCounter_CONTROL |= SingleWireCommCounter_ONESHOT;
            #endif /* (SingleWireCommCounter_RunModeUsed != 0x0u) */
            
            /* Set the IRQ to use the status register interrupts */
            SingleWireCommCounter_CONTROL2 |= SingleWireCommCounter_CTRL2_IRQ_SEL;
            
            /* Clear and Set SYNCTC and SYNCCMP bits of RT1 register */
            SingleWireCommCounter_RT1 &= ((uint8)(~SingleWireCommCounter_RT1_MASK));
            SingleWireCommCounter_RT1 |= SingleWireCommCounter_SYNC;     
                    
            /*Enable DSI Sync all all inputs of the Timer*/
            SingleWireCommCounter_RT1 &= ((uint8)(~SingleWireCommCounter_SYNCDSI_MASK));
            SingleWireCommCounter_RT1 |= SingleWireCommCounter_SYNCDSI_EN;

        #else
            #if(!SingleWireCommCounter_ControlRegRemoved)
            /* Set the default compare mode defined in the parameter */
            ctrl = SingleWireCommCounter_CONTROL & ((uint8)(~SingleWireCommCounter_CTRL_CMPMODE_MASK));
            SingleWireCommCounter_CONTROL = ctrl | SingleWireCommCounter_DEFAULT_COMPARE_MODE;
            
            /* Set the default capture mode defined in the parameter */
            ctrl = SingleWireCommCounter_CONTROL & ((uint8)(~SingleWireCommCounter_CTRL_CAPMODE_MASK));
            
            #if( 0 != SingleWireCommCounter_CAPTURE_MODE_CONF)
                SingleWireCommCounter_CONTROL = ctrl | SingleWireCommCounter_DEFAULT_CAPTURE_MODE;
            #else
                SingleWireCommCounter_CONTROL = ctrl;
            #endif /* 0 != SingleWireCommCounter_CAPTURE_MODE */ 
            
            #endif /* (!SingleWireCommCounter_ControlRegRemoved) */
        #endif /* (SingleWireCommCounter_UsingFixedFunction) */
        
        /* Clear all data in the FIFO's */
        #if (!SingleWireCommCounter_UsingFixedFunction)
            SingleWireCommCounter_ClearFIFO();
        #endif /* (!SingleWireCommCounter_UsingFixedFunction) */
        
        /* Set Initial values from Configuration */
        SingleWireCommCounter_WritePeriod(SingleWireCommCounter_INIT_PERIOD_VALUE);
        #if (!(SingleWireCommCounter_UsingFixedFunction && (CY_PSOC5A)))
            SingleWireCommCounter_WriteCounter(SingleWireCommCounter_INIT_COUNTER_VALUE);
        #endif /* (!(SingleWireCommCounter_UsingFixedFunction && (CY_PSOC5A))) */
        SingleWireCommCounter_SetInterruptMode(SingleWireCommCounter_INIT_INTERRUPTS_MASK);
        
        #if (!SingleWireCommCounter_UsingFixedFunction)
            /* Read the status register to clear the unwanted interrupts */
            (void)SingleWireCommCounter_ReadStatusRegister();
            /* Set the compare value (only available to non-fixed function implementation */
            SingleWireCommCounter_WriteCompare(SingleWireCommCounter_INIT_COMPARE_VALUE);
            /* Use the interrupt output of the status register for IRQ output */
            
            /* CyEnterCriticalRegion and CyExitCriticalRegion are used to mark following region critical*/
            /* Enter Critical Region*/
            SingleWireCommCounter_interruptState = CyEnterCriticalSection();
            
            SingleWireCommCounter_STATUS_AUX_CTRL |= SingleWireCommCounter_STATUS_ACTL_INT_EN_MASK;
            
            /* Exit Critical Region*/
            CyExitCriticalSection(SingleWireCommCounter_interruptState);
            
        #endif /* (!SingleWireCommCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: SingleWireCommCounter_Enable
********************************************************************************
* Summary:
*     Enable the Counter
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: 
*   If the Enable mode is set to Hardware only then this function has no effect 
*   on the operation of the counter.
*
*******************************************************************************/
void SingleWireCommCounter_Enable(void) 
{
    /* Globally Enable the Fixed Function Block chosen */
    #if (SingleWireCommCounter_UsingFixedFunction)
        SingleWireCommCounter_GLOBAL_ENABLE |= SingleWireCommCounter_BLOCK_EN_MASK;
        SingleWireCommCounter_GLOBAL_STBY_ENABLE |= SingleWireCommCounter_BLOCK_STBY_EN_MASK;
    #endif /* (SingleWireCommCounter_UsingFixedFunction) */  
        
    /* Enable the counter from the control register  */
    /* If Fixed Function then make sure Mode is set correctly */
    /* else make sure reset is clear */
    #if(!SingleWireCommCounter_ControlRegRemoved || SingleWireCommCounter_UsingFixedFunction)
        SingleWireCommCounter_CONTROL |= SingleWireCommCounter_CTRL_ENABLE;                
    #endif /* (!SingleWireCommCounter_ControlRegRemoved || SingleWireCommCounter_UsingFixedFunction) */
    
}


/*******************************************************************************
* Function Name: SingleWireCommCounter_Start
********************************************************************************
* Summary:
*  Enables the counter for operation 
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  SingleWireCommCounter_initVar: Is modified when this function is called for the  
*   first time. Is used to ensure that initialization happens only once.
*
*******************************************************************************/
void SingleWireCommCounter_Start(void) 
{
    if(SingleWireCommCounter_initVar == 0u)
    {
        SingleWireCommCounter_Init();
        
        SingleWireCommCounter_initVar = 1u; /* Clear this bit for Initialization */        
    }
    
    /* Enable the Counter */
    SingleWireCommCounter_Enable();        
}


/*******************************************************************************
* Function Name: SingleWireCommCounter_Stop
********************************************************************************
* Summary:
* Halts the counter, but does not change any modes or disable interrupts.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: If the Enable mode is set to Hardware only then this function
*               has no effect on the operation of the counter.
*
*******************************************************************************/
void SingleWireCommCounter_Stop(void) 
{
    /* Disable Counter */
    #if(!SingleWireCommCounter_ControlRegRemoved || SingleWireCommCounter_UsingFixedFunction)
        SingleWireCommCounter_CONTROL &= ((uint8)(~SingleWireCommCounter_CTRL_ENABLE));        
    #endif /* (!SingleWireCommCounter_ControlRegRemoved || SingleWireCommCounter_UsingFixedFunction) */
    
    /* Globally disable the Fixed Function Block chosen */
    #if (SingleWireCommCounter_UsingFixedFunction)
        SingleWireCommCounter_GLOBAL_ENABLE &= ((uint8)(~SingleWireCommCounter_BLOCK_EN_MASK));
        SingleWireCommCounter_GLOBAL_STBY_ENABLE &= ((uint8)(~SingleWireCommCounter_BLOCK_STBY_EN_MASK));
    #endif /* (SingleWireCommCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: SingleWireCommCounter_SetInterruptMode
********************************************************************************
* Summary:
* Configures which interrupt sources are enabled to generate the final interrupt
*
* Parameters:  
*  InterruptsMask: This parameter is an or'd collection of the status bits
*                   which will be allowed to generate the counters interrupt.   
*
* Return: 
*  void
*
*******************************************************************************/
void SingleWireCommCounter_SetInterruptMode(uint8 interruptsMask) 
{
    SingleWireCommCounter_STATUS_MASK = interruptsMask;
}


/*******************************************************************************
* Function Name: SingleWireCommCounter_ReadStatusRegister
********************************************************************************
* Summary:
*   Reads the status register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the status register
*
* Side Effects:
*   Status register bits may be clear on read. 
*
*******************************************************************************/
uint8   SingleWireCommCounter_ReadStatusRegister(void) 
{
    return SingleWireCommCounter_STATUS;
}


#if(!SingleWireCommCounter_ControlRegRemoved)
/*******************************************************************************
* Function Name: SingleWireCommCounter_ReadControlRegister
********************************************************************************
* Summary:
*   Reads the control register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
uint8   SingleWireCommCounter_ReadControlRegister(void) 
{
    return SingleWireCommCounter_CONTROL;
}


/*******************************************************************************
* Function Name: SingleWireCommCounter_WriteControlRegister
********************************************************************************
* Summary:
*   Sets the bit-field of the control register.  This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
void    SingleWireCommCounter_WriteControlRegister(uint8 control) 
{
    SingleWireCommCounter_CONTROL = control;
}

#endif  /* (!SingleWireCommCounter_ControlRegRemoved) */


#if (!(SingleWireCommCounter_UsingFixedFunction && (CY_PSOC5A)))
/*******************************************************************************
* Function Name: SingleWireCommCounter_WriteCounter
********************************************************************************
* Summary:
*   This funtion is used to set the counter to a specific value
*
* Parameters:  
*  counter:  New counter value. 
*
* Return: 
*  void 
*
*******************************************************************************/
void SingleWireCommCounter_WriteCounter(uint8 counter) \
                                   
{
    #if(SingleWireCommCounter_UsingFixedFunction)
        /* assert if block is already enabled */
        CYASSERT (0u == (SingleWireCommCounter_GLOBAL_ENABLE & SingleWireCommCounter_BLOCK_EN_MASK));
        /* If block is disabled, enable it and then write the counter */
        SingleWireCommCounter_GLOBAL_ENABLE |= SingleWireCommCounter_BLOCK_EN_MASK;
        CY_SET_REG16(SingleWireCommCounter_COUNTER_LSB_PTR, (uint16)counter);
        SingleWireCommCounter_GLOBAL_ENABLE &= ((uint8)(~SingleWireCommCounter_BLOCK_EN_MASK));
    #else
        CY_SET_REG8(SingleWireCommCounter_COUNTER_LSB_PTR, counter);
    #endif /* (SingleWireCommCounter_UsingFixedFunction) */
}
#endif /* (!(SingleWireCommCounter_UsingFixedFunction && (CY_PSOC5A))) */


/*******************************************************************************
* Function Name: SingleWireCommCounter_ReadCounter
********************************************************************************
* Summary:
* Returns the current value of the counter.  It doesn't matter
* if the counter is enabled or running.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint8) The present value of the counter.
*
*******************************************************************************/
uint8 SingleWireCommCounter_ReadCounter(void) 
{
    /* Force capture by reading Accumulator */
    /* Must first do a software capture to be able to read the counter */
    /* It is up to the user code to make sure there isn't already captured data in the FIFO */
    #if(SingleWireCommCounter_UsingFixedFunction)
		(void)CY_GET_REG16(SingleWireCommCounter_COUNTER_LSB_PTR);
	#else
		(void)CY_GET_REG8(SingleWireCommCounter_COUNTER_LSB_PTR_8BIT);
	#endif/* (SingleWireCommCounter_UsingFixedFunction) */
    
    /* Read the data from the FIFO (or capture register for Fixed Function)*/
    #if(SingleWireCommCounter_UsingFixedFunction)
        return ((uint8)CY_GET_REG16(SingleWireCommCounter_STATICCOUNT_LSB_PTR));
    #else
        return (CY_GET_REG8(SingleWireCommCounter_STATICCOUNT_LSB_PTR));
    #endif /* (SingleWireCommCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: SingleWireCommCounter_ReadCapture
********************************************************************************
* Summary:
*   This function returns the last value captured.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) Present Capture value.
*
*******************************************************************************/
uint8 SingleWireCommCounter_ReadCapture(void) 
{
    #if(SingleWireCommCounter_UsingFixedFunction)
        return ((uint8)CY_GET_REG16(SingleWireCommCounter_STATICCOUNT_LSB_PTR));
    #else
        return (CY_GET_REG8(SingleWireCommCounter_STATICCOUNT_LSB_PTR));
    #endif /* (SingleWireCommCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: SingleWireCommCounter_WritePeriod
********************************************************************************
* Summary:
* Changes the period of the counter.  The new period 
* will be loaded the next time terminal count is detected.
*
* Parameters:  
*  period: (uint8) A value of 0 will result in
*         the counter remaining at zero.  
*
* Return: 
*  void
*
*******************************************************************************/
void SingleWireCommCounter_WritePeriod(uint8 period) 
{
    #if(SingleWireCommCounter_UsingFixedFunction)
        CY_SET_REG16(SingleWireCommCounter_PERIOD_LSB_PTR,(uint16)period);
    #else
        CY_SET_REG8(SingleWireCommCounter_PERIOD_LSB_PTR, period);
    #endif /* (SingleWireCommCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: SingleWireCommCounter_ReadPeriod
********************************************************************************
* Summary:
* Reads the current period value without affecting counter operation.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint8) Present period value.
*
*******************************************************************************/
uint8 SingleWireCommCounter_ReadPeriod(void) 
{
    #if(SingleWireCommCounter_UsingFixedFunction)
        return ((uint8)CY_GET_REG16(SingleWireCommCounter_PERIOD_LSB_PTR));
    #else
        return (CY_GET_REG8(SingleWireCommCounter_PERIOD_LSB_PTR));
    #endif /* (SingleWireCommCounter_UsingFixedFunction) */
}


#if (!SingleWireCommCounter_UsingFixedFunction)
/*******************************************************************************
* Function Name: SingleWireCommCounter_WriteCompare
********************************************************************************
* Summary:
* Changes the compare value.  The compare output will 
* reflect the new value on the next UDB clock.  The compare output will be 
* driven high when the present counter value compares true based on the 
* configured compare mode setting. 
*
* Parameters:  
*  Compare:  New compare value. 
*
* Return: 
*  void
*
*******************************************************************************/
void SingleWireCommCounter_WriteCompare(uint8 compare) \
                                   
{
    #if(SingleWireCommCounter_UsingFixedFunction)
        CY_SET_REG16(SingleWireCommCounter_COMPARE_LSB_PTR, (uint16)compare);
    #else
        CY_SET_REG8(SingleWireCommCounter_COMPARE_LSB_PTR, compare);
    #endif /* (SingleWireCommCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: SingleWireCommCounter_ReadCompare
********************************************************************************
* Summary:
* Returns the compare value.
*
* Parameters:  
*  void:
*
* Return: 
*  (uint8) Present compare value.
*
*******************************************************************************/
uint8 SingleWireCommCounter_ReadCompare(void) 
{
    return (CY_GET_REG8(SingleWireCommCounter_COMPARE_LSB_PTR));
}


#if (SingleWireCommCounter_COMPARE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: SingleWireCommCounter_SetCompareMode
********************************************************************************
* Summary:
*  Sets the software controlled Compare Mode.
*
* Parameters:
*  compareMode:  Compare Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void SingleWireCommCounter_SetCompareMode(uint8 compareMode) 
{
    /* Clear the compare mode bits in the control register */
    SingleWireCommCounter_CONTROL &= ((uint8)(~SingleWireCommCounter_CTRL_CMPMODE_MASK));
    
    /* Write the new setting */
    SingleWireCommCounter_CONTROL |= compareMode;
}
#endif  /* (SingleWireCommCounter_COMPARE_MODE_SOFTWARE) */


#if (SingleWireCommCounter_CAPTURE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: SingleWireCommCounter_SetCaptureMode
********************************************************************************
* Summary:
*  Sets the software controlled Capture Mode.
*
* Parameters:
*  captureMode:  Capture Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void SingleWireCommCounter_SetCaptureMode(uint8 captureMode) 
{
    /* Clear the capture mode bits in the control register */
    SingleWireCommCounter_CONTROL &= ((uint8)(~SingleWireCommCounter_CTRL_CAPMODE_MASK));
    
    /* Write the new setting */
    SingleWireCommCounter_CONTROL |= ((uint8)((uint8)captureMode << SingleWireCommCounter_CTRL_CAPMODE0_SHIFT));
}
#endif  /* (SingleWireCommCounter_CAPTURE_MODE_SOFTWARE) */


/*******************************************************************************
* Function Name: SingleWireCommCounter_ClearFIFO
********************************************************************************
* Summary:
*   This function clears all capture data from the capture FIFO
*
* Parameters:  
*  void:
*
* Return: 
*  None
*
*******************************************************************************/
void SingleWireCommCounter_ClearFIFO(void) 
{

    while(0u != (SingleWireCommCounter_ReadStatusRegister() & SingleWireCommCounter_STATUS_FIFONEMP))
    {
        (void)SingleWireCommCounter_ReadCapture();
    }

}
#endif  /* (!SingleWireCommCounter_UsingFixedFunction) */


/* [] END OF FILE */

