/*******************************************************************************
* File Name: SingleWireCommCounter_PM.c  
* Version 3.0
*
*  Description:
*    This file provides the power management source code to API for the
*    Counter.  
*
*   Note:
*     None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "SingleWireCommCounter.h"

static SingleWireCommCounter_backupStruct SingleWireCommCounter_backup;


/*******************************************************************************
* Function Name: SingleWireCommCounter_SaveConfig
********************************************************************************
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  SingleWireCommCounter_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
*******************************************************************************/
void SingleWireCommCounter_SaveConfig(void) 
{
    #if (!SingleWireCommCounter_UsingFixedFunction)

        SingleWireCommCounter_backup.CounterUdb = SingleWireCommCounter_ReadCounter();

        #if(!SingleWireCommCounter_ControlRegRemoved)
            SingleWireCommCounter_backup.CounterControlRegister = SingleWireCommCounter_ReadControlRegister();
        #endif /* (!SingleWireCommCounter_ControlRegRemoved) */

    #endif /* (!SingleWireCommCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: SingleWireCommCounter_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  SingleWireCommCounter_backup:  Variables of this global structure are used to 
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void SingleWireCommCounter_RestoreConfig(void) 
{      
    #if (!SingleWireCommCounter_UsingFixedFunction)

       SingleWireCommCounter_WriteCounter(SingleWireCommCounter_backup.CounterUdb);

        #if(!SingleWireCommCounter_ControlRegRemoved)
            SingleWireCommCounter_WriteControlRegister(SingleWireCommCounter_backup.CounterControlRegister);
        #endif /* (!SingleWireCommCounter_ControlRegRemoved) */

    #endif /* (!SingleWireCommCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: SingleWireCommCounter_Sleep
********************************************************************************
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  SingleWireCommCounter_backup.enableState:  Is modified depending on the enable 
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void SingleWireCommCounter_Sleep(void) 
{
    #if(!SingleWireCommCounter_ControlRegRemoved)
        /* Save Counter's enable state */
        if(SingleWireCommCounter_CTRL_ENABLE == (SingleWireCommCounter_CONTROL & SingleWireCommCounter_CTRL_ENABLE))
        {
            /* Counter is enabled */
            SingleWireCommCounter_backup.CounterEnableState = 1u;
        }
        else
        {
            /* Counter is disabled */
            SingleWireCommCounter_backup.CounterEnableState = 0u;
        }
    #else
        SingleWireCommCounter_backup.CounterEnableState = 1u;
        if(SingleWireCommCounter_backup.CounterEnableState != 0u)
        {
            SingleWireCommCounter_backup.CounterEnableState = 0u;
        }
    #endif /* (!SingleWireCommCounter_ControlRegRemoved) */
    
    SingleWireCommCounter_Stop();
    SingleWireCommCounter_SaveConfig();
}


/*******************************************************************************
* Function Name: SingleWireCommCounter_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  SingleWireCommCounter_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void SingleWireCommCounter_Wakeup(void) 
{
    SingleWireCommCounter_RestoreConfig();
    #if(!SingleWireCommCounter_ControlRegRemoved)
        if(SingleWireCommCounter_backup.CounterEnableState == 1u)
        {
            /* Enable Counter's operation */
            SingleWireCommCounter_Enable();
        } /* Do nothing if Counter was disabled before */    
    #endif /* (!SingleWireCommCounter_ControlRegRemoved) */
    
}


/* [] END OF FILE */
