/*******************************************************************************
* File Name: SingleWireCommTimer_PM.c
* Version 2.70
*
*  Description:
*     This file provides the power management source code to API for the
*     Timer.
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "SingleWireCommTimer.h"

static SingleWireCommTimer_backupStruct SingleWireCommTimer_backup;


/*******************************************************************************
* Function Name: SingleWireCommTimer_SaveConfig
********************************************************************************
*
* Summary:
*     Save the current user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  SingleWireCommTimer_backup:  Variables of this global structure are modified to
*  store the values of non retention configuration registers when Sleep() API is
*  called.
*
*******************************************************************************/
void SingleWireCommTimer_SaveConfig(void) 
{
    #if (!SingleWireCommTimer_UsingFixedFunction)
        SingleWireCommTimer_backup.TimerUdb = SingleWireCommTimer_ReadCounter();
        SingleWireCommTimer_backup.InterruptMaskValue = SingleWireCommTimer_STATUS_MASK;
        #if (SingleWireCommTimer_UsingHWCaptureCounter)
            SingleWireCommTimer_backup.TimerCaptureCounter = SingleWireCommTimer_ReadCaptureCount();
        #endif /* Back Up capture counter register  */

        #if(!SingleWireCommTimer_UDB_CONTROL_REG_REMOVED)
            SingleWireCommTimer_backup.TimerControlRegister = SingleWireCommTimer_ReadControlRegister();
        #endif /* Backup the enable state of the Timer component */
    #endif /* Backup non retention registers in UDB implementation. All fixed function registers are retention */
}


/*******************************************************************************
* Function Name: SingleWireCommTimer_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  SingleWireCommTimer_backup:  Variables of this global structure are used to
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void SingleWireCommTimer_RestoreConfig(void) 
{   
    #if (!SingleWireCommTimer_UsingFixedFunction)

        SingleWireCommTimer_WriteCounter(SingleWireCommTimer_backup.TimerUdb);
        SingleWireCommTimer_STATUS_MASK =SingleWireCommTimer_backup.InterruptMaskValue;
        #if (SingleWireCommTimer_UsingHWCaptureCounter)
            SingleWireCommTimer_SetCaptureCount(SingleWireCommTimer_backup.TimerCaptureCounter);
        #endif /* Restore Capture counter register*/

        #if(!SingleWireCommTimer_UDB_CONTROL_REG_REMOVED)
            SingleWireCommTimer_WriteControlRegister(SingleWireCommTimer_backup.TimerControlRegister);
        #endif /* Restore the enable state of the Timer component */
    #endif /* Restore non retention registers in the UDB implementation only */
}


/*******************************************************************************
* Function Name: SingleWireCommTimer_Sleep
********************************************************************************
*
* Summary:
*     Stop and Save the user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  SingleWireCommTimer_backup.TimerEnableState:  Is modified depending on the
*  enable state of the block before entering sleep mode.
*
*******************************************************************************/
void SingleWireCommTimer_Sleep(void) 
{
    #if(!SingleWireCommTimer_UDB_CONTROL_REG_REMOVED)
        /* Save Counter's enable state */
        if(SingleWireCommTimer_CTRL_ENABLE == (SingleWireCommTimer_CONTROL & SingleWireCommTimer_CTRL_ENABLE))
        {
            /* Timer is enabled */
            SingleWireCommTimer_backup.TimerEnableState = 1u;
        }
        else
        {
            /* Timer is disabled */
            SingleWireCommTimer_backup.TimerEnableState = 0u;
        }
    #endif /* Back up enable state from the Timer control register */
    SingleWireCommTimer_Stop();
    SingleWireCommTimer_SaveConfig();
}


/*******************************************************************************
* Function Name: SingleWireCommTimer_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  SingleWireCommTimer_backup.enableState:  Is used to restore the enable state of
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void SingleWireCommTimer_Wakeup(void) 
{
    SingleWireCommTimer_RestoreConfig();
    #if(!SingleWireCommTimer_UDB_CONTROL_REG_REMOVED)
        if(SingleWireCommTimer_backup.TimerEnableState == 1u)
        {     /* Enable Timer's operation */
                SingleWireCommTimer_Enable();
        } /* Do nothing if Timer was disabled before */
    #endif /* Remove this code section if Control register is removed */
}


/* [] END OF FILE */
