/*******************************************************************************
* File Name: SpeedMsgTimer_PM.c
* Version 2.70
*
*  Description:
*     This file provides the power management source code to API for the
*     Timer.
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "SpeedMsgTimer.h"

static SpeedMsgTimer_backupStruct SpeedMsgTimer_backup;


/*******************************************************************************
* Function Name: SpeedMsgTimer_SaveConfig
********************************************************************************
*
* Summary:
*     Save the current user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  SpeedMsgTimer_backup:  Variables of this global structure are modified to
*  store the values of non retention configuration registers when Sleep() API is
*  called.
*
*******************************************************************************/
void SpeedMsgTimer_SaveConfig(void) 
{
    #if (!SpeedMsgTimer_UsingFixedFunction)
        SpeedMsgTimer_backup.TimerUdb = SpeedMsgTimer_ReadCounter();
        SpeedMsgTimer_backup.InterruptMaskValue = SpeedMsgTimer_STATUS_MASK;
        #if (SpeedMsgTimer_UsingHWCaptureCounter)
            SpeedMsgTimer_backup.TimerCaptureCounter = SpeedMsgTimer_ReadCaptureCount();
        #endif /* Back Up capture counter register  */

        #if(!SpeedMsgTimer_UDB_CONTROL_REG_REMOVED)
            SpeedMsgTimer_backup.TimerControlRegister = SpeedMsgTimer_ReadControlRegister();
        #endif /* Backup the enable state of the Timer component */
    #endif /* Backup non retention registers in UDB implementation. All fixed function registers are retention */
}


/*******************************************************************************
* Function Name: SpeedMsgTimer_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  SpeedMsgTimer_backup:  Variables of this global structure are used to
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void SpeedMsgTimer_RestoreConfig(void) 
{   
    #if (!SpeedMsgTimer_UsingFixedFunction)

        SpeedMsgTimer_WriteCounter(SpeedMsgTimer_backup.TimerUdb);
        SpeedMsgTimer_STATUS_MASK =SpeedMsgTimer_backup.InterruptMaskValue;
        #if (SpeedMsgTimer_UsingHWCaptureCounter)
            SpeedMsgTimer_SetCaptureCount(SpeedMsgTimer_backup.TimerCaptureCounter);
        #endif /* Restore Capture counter register*/

        #if(!SpeedMsgTimer_UDB_CONTROL_REG_REMOVED)
            SpeedMsgTimer_WriteControlRegister(SpeedMsgTimer_backup.TimerControlRegister);
        #endif /* Restore the enable state of the Timer component */
    #endif /* Restore non retention registers in the UDB implementation only */
}


/*******************************************************************************
* Function Name: SpeedMsgTimer_Sleep
********************************************************************************
*
* Summary:
*     Stop and Save the user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  SpeedMsgTimer_backup.TimerEnableState:  Is modified depending on the
*  enable state of the block before entering sleep mode.
*
*******************************************************************************/
void SpeedMsgTimer_Sleep(void) 
{
    #if(!SpeedMsgTimer_UDB_CONTROL_REG_REMOVED)
        /* Save Counter's enable state */
        if(SpeedMsgTimer_CTRL_ENABLE == (SpeedMsgTimer_CONTROL & SpeedMsgTimer_CTRL_ENABLE))
        {
            /* Timer is enabled */
            SpeedMsgTimer_backup.TimerEnableState = 1u;
        }
        else
        {
            /* Timer is disabled */
            SpeedMsgTimer_backup.TimerEnableState = 0u;
        }
    #endif /* Back up enable state from the Timer control register */
    SpeedMsgTimer_Stop();
    SpeedMsgTimer_SaveConfig();
}


/*******************************************************************************
* Function Name: SpeedMsgTimer_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  SpeedMsgTimer_backup.enableState:  Is used to restore the enable state of
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void SpeedMsgTimer_Wakeup(void) 
{
    SpeedMsgTimer_RestoreConfig();
    #if(!SpeedMsgTimer_UDB_CONTROL_REG_REMOVED)
        if(SpeedMsgTimer_backup.TimerEnableState == 1u)
        {     /* Enable Timer's operation */
                SpeedMsgTimer_Enable();
        } /* Do nothing if Timer was disabled before */
    #endif /* Remove this code section if Control register is removed */
}


/* [] END OF FILE */
