/*******************************************************************************
* File Name: TestCounter.c  
* Version 3.0
*
*  Description:
*     The Counter component consists of a 8, 16, 24 or 32-bit counter with
*     a selectable period between 2 and 2^Width - 1.  
*
*   Note:
*     None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "TestCounter.h"

uint8 TestCounter_initVar = 0u;


/*******************************************************************************
* Function Name: TestCounter_Init
********************************************************************************
* Summary:
*     Initialize to the schematic state
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
*******************************************************************************/
void TestCounter_Init(void) 
{
        #if (!TestCounter_UsingFixedFunction && !TestCounter_ControlRegRemoved)
            uint8 ctrl;
        #endif /* (!TestCounter_UsingFixedFunction && !TestCounter_ControlRegRemoved) */
        
        #if(!TestCounter_UsingFixedFunction) 
            /* Interrupt State Backup for Critical Region*/
            uint8 TestCounter_interruptState;
        #endif /* (!TestCounter_UsingFixedFunction) */
        
        #if (TestCounter_UsingFixedFunction)
            /* Clear all bits but the enable bit (if it's already set for Timer operation */
            TestCounter_CONTROL &= TestCounter_CTRL_ENABLE;
            
            /* Clear the mode bits for continuous run mode */
            #if (CY_PSOC5A)
                TestCounter_CONTROL2 &= ((uint8)(~TestCounter_CTRL_MODE_MASK));
            #endif /* (CY_PSOC5A) */
            #if (CY_PSOC3 || CY_PSOC5LP)
                TestCounter_CONTROL3 &= ((uint8)(~TestCounter_CTRL_MODE_MASK));                
            #endif /* (CY_PSOC3 || CY_PSOC5LP) */
            /* Check if One Shot mode is enabled i.e. RunMode !=0*/
            #if (TestCounter_RunModeUsed != 0x0u)
                /* Set 3rd bit of Control register to enable one shot mode */
                TestCounter_CONTROL |= TestCounter_ONESHOT;
            #endif /* (TestCounter_RunModeUsed != 0x0u) */
            
            /* Set the IRQ to use the status register interrupts */
            TestCounter_CONTROL2 |= TestCounter_CTRL2_IRQ_SEL;
            
            /* Clear and Set SYNCTC and SYNCCMP bits of RT1 register */
            TestCounter_RT1 &= ((uint8)(~TestCounter_RT1_MASK));
            TestCounter_RT1 |= TestCounter_SYNC;     
                    
            /*Enable DSI Sync all all inputs of the Timer*/
            TestCounter_RT1 &= ((uint8)(~TestCounter_SYNCDSI_MASK));
            TestCounter_RT1 |= TestCounter_SYNCDSI_EN;

        #else
            #if(!TestCounter_ControlRegRemoved)
            /* Set the default compare mode defined in the parameter */
            ctrl = TestCounter_CONTROL & ((uint8)(~TestCounter_CTRL_CMPMODE_MASK));
            TestCounter_CONTROL = ctrl | TestCounter_DEFAULT_COMPARE_MODE;
            
            /* Set the default capture mode defined in the parameter */
            ctrl = TestCounter_CONTROL & ((uint8)(~TestCounter_CTRL_CAPMODE_MASK));
            
            #if( 0 != TestCounter_CAPTURE_MODE_CONF)
                TestCounter_CONTROL = ctrl | TestCounter_DEFAULT_CAPTURE_MODE;
            #else
                TestCounter_CONTROL = ctrl;
            #endif /* 0 != TestCounter_CAPTURE_MODE */ 
            
            #endif /* (!TestCounter_ControlRegRemoved) */
        #endif /* (TestCounter_UsingFixedFunction) */
        
        /* Clear all data in the FIFO's */
        #if (!TestCounter_UsingFixedFunction)
            TestCounter_ClearFIFO();
        #endif /* (!TestCounter_UsingFixedFunction) */
        
        /* Set Initial values from Configuration */
        TestCounter_WritePeriod(TestCounter_INIT_PERIOD_VALUE);
        #if (!(TestCounter_UsingFixedFunction && (CY_PSOC5A)))
            TestCounter_WriteCounter(TestCounter_INIT_COUNTER_VALUE);
        #endif /* (!(TestCounter_UsingFixedFunction && (CY_PSOC5A))) */
        TestCounter_SetInterruptMode(TestCounter_INIT_INTERRUPTS_MASK);
        
        #if (!TestCounter_UsingFixedFunction)
            /* Read the status register to clear the unwanted interrupts */
            (void)TestCounter_ReadStatusRegister();
            /* Set the compare value (only available to non-fixed function implementation */
            TestCounter_WriteCompare(TestCounter_INIT_COMPARE_VALUE);
            /* Use the interrupt output of the status register for IRQ output */
            
            /* CyEnterCriticalRegion and CyExitCriticalRegion are used to mark following region critical*/
            /* Enter Critical Region*/
            TestCounter_interruptState = CyEnterCriticalSection();
            
            TestCounter_STATUS_AUX_CTRL |= TestCounter_STATUS_ACTL_INT_EN_MASK;
            
            /* Exit Critical Region*/
            CyExitCriticalSection(TestCounter_interruptState);
            
        #endif /* (!TestCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: TestCounter_Enable
********************************************************************************
* Summary:
*     Enable the Counter
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: 
*   If the Enable mode is set to Hardware only then this function has no effect 
*   on the operation of the counter.
*
*******************************************************************************/
void TestCounter_Enable(void) 
{
    /* Globally Enable the Fixed Function Block chosen */
    #if (TestCounter_UsingFixedFunction)
        TestCounter_GLOBAL_ENABLE |= TestCounter_BLOCK_EN_MASK;
        TestCounter_GLOBAL_STBY_ENABLE |= TestCounter_BLOCK_STBY_EN_MASK;
    #endif /* (TestCounter_UsingFixedFunction) */  
        
    /* Enable the counter from the control register  */
    /* If Fixed Function then make sure Mode is set correctly */
    /* else make sure reset is clear */
    #if(!TestCounter_ControlRegRemoved || TestCounter_UsingFixedFunction)
        TestCounter_CONTROL |= TestCounter_CTRL_ENABLE;                
    #endif /* (!TestCounter_ControlRegRemoved || TestCounter_UsingFixedFunction) */
    
}


/*******************************************************************************
* Function Name: TestCounter_Start
********************************************************************************
* Summary:
*  Enables the counter for operation 
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  TestCounter_initVar: Is modified when this function is called for the  
*   first time. Is used to ensure that initialization happens only once.
*
*******************************************************************************/
void TestCounter_Start(void) 
{
    if(TestCounter_initVar == 0u)
    {
        TestCounter_Init();
        
        TestCounter_initVar = 1u; /* Clear this bit for Initialization */        
    }
    
    /* Enable the Counter */
    TestCounter_Enable();        
}


/*******************************************************************************
* Function Name: TestCounter_Stop
********************************************************************************
* Summary:
* Halts the counter, but does not change any modes or disable interrupts.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: If the Enable mode is set to Hardware only then this function
*               has no effect on the operation of the counter.
*
*******************************************************************************/
void TestCounter_Stop(void) 
{
    /* Disable Counter */
    #if(!TestCounter_ControlRegRemoved || TestCounter_UsingFixedFunction)
        TestCounter_CONTROL &= ((uint8)(~TestCounter_CTRL_ENABLE));        
    #endif /* (!TestCounter_ControlRegRemoved || TestCounter_UsingFixedFunction) */
    
    /* Globally disable the Fixed Function Block chosen */
    #if (TestCounter_UsingFixedFunction)
        TestCounter_GLOBAL_ENABLE &= ((uint8)(~TestCounter_BLOCK_EN_MASK));
        TestCounter_GLOBAL_STBY_ENABLE &= ((uint8)(~TestCounter_BLOCK_STBY_EN_MASK));
    #endif /* (TestCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: TestCounter_SetInterruptMode
********************************************************************************
* Summary:
* Configures which interrupt sources are enabled to generate the final interrupt
*
* Parameters:  
*  InterruptsMask: This parameter is an or'd collection of the status bits
*                   which will be allowed to generate the counters interrupt.   
*
* Return: 
*  void
*
*******************************************************************************/
void TestCounter_SetInterruptMode(uint8 interruptsMask) 
{
    TestCounter_STATUS_MASK = interruptsMask;
}


/*******************************************************************************
* Function Name: TestCounter_ReadStatusRegister
********************************************************************************
* Summary:
*   Reads the status register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the status register
*
* Side Effects:
*   Status register bits may be clear on read. 
*
*******************************************************************************/
uint8   TestCounter_ReadStatusRegister(void) 
{
    return TestCounter_STATUS;
}


#if(!TestCounter_ControlRegRemoved)
/*******************************************************************************
* Function Name: TestCounter_ReadControlRegister
********************************************************************************
* Summary:
*   Reads the control register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
uint8   TestCounter_ReadControlRegister(void) 
{
    return TestCounter_CONTROL;
}


/*******************************************************************************
* Function Name: TestCounter_WriteControlRegister
********************************************************************************
* Summary:
*   Sets the bit-field of the control register.  This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
void    TestCounter_WriteControlRegister(uint8 control) 
{
    TestCounter_CONTROL = control;
}

#endif  /* (!TestCounter_ControlRegRemoved) */


#if (!(TestCounter_UsingFixedFunction && (CY_PSOC5A)))
/*******************************************************************************
* Function Name: TestCounter_WriteCounter
********************************************************************************
* Summary:
*   This funtion is used to set the counter to a specific value
*
* Parameters:  
*  counter:  New counter value. 
*
* Return: 
*  void 
*
*******************************************************************************/
void TestCounter_WriteCounter(uint16 counter) \
                                   
{
    #if(TestCounter_UsingFixedFunction)
        /* assert if block is already enabled */
        CYASSERT (0u == (TestCounter_GLOBAL_ENABLE & TestCounter_BLOCK_EN_MASK));
        /* If block is disabled, enable it and then write the counter */
        TestCounter_GLOBAL_ENABLE |= TestCounter_BLOCK_EN_MASK;
        CY_SET_REG16(TestCounter_COUNTER_LSB_PTR, (uint16)counter);
        TestCounter_GLOBAL_ENABLE &= ((uint8)(~TestCounter_BLOCK_EN_MASK));
    #else
        CY_SET_REG16(TestCounter_COUNTER_LSB_PTR, counter);
    #endif /* (TestCounter_UsingFixedFunction) */
}
#endif /* (!(TestCounter_UsingFixedFunction && (CY_PSOC5A))) */


/*******************************************************************************
* Function Name: TestCounter_ReadCounter
********************************************************************************
* Summary:
* Returns the current value of the counter.  It doesn't matter
* if the counter is enabled or running.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint16) The present value of the counter.
*
*******************************************************************************/
uint16 TestCounter_ReadCounter(void) 
{
    /* Force capture by reading Accumulator */
    /* Must first do a software capture to be able to read the counter */
    /* It is up to the user code to make sure there isn't already captured data in the FIFO */
    #if(TestCounter_UsingFixedFunction)
		(void)CY_GET_REG16(TestCounter_COUNTER_LSB_PTR);
	#else
		(void)CY_GET_REG8(TestCounter_COUNTER_LSB_PTR_8BIT);
	#endif/* (TestCounter_UsingFixedFunction) */
    
    /* Read the data from the FIFO (or capture register for Fixed Function)*/
    #if(TestCounter_UsingFixedFunction)
        return ((uint16)CY_GET_REG16(TestCounter_STATICCOUNT_LSB_PTR));
    #else
        return (CY_GET_REG16(TestCounter_STATICCOUNT_LSB_PTR));
    #endif /* (TestCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: TestCounter_ReadCapture
********************************************************************************
* Summary:
*   This function returns the last value captured.
*
* Parameters:  
*  void
*
* Return: 
*  (uint16) Present Capture value.
*
*******************************************************************************/
uint16 TestCounter_ReadCapture(void) 
{
    #if(TestCounter_UsingFixedFunction)
        return ((uint16)CY_GET_REG16(TestCounter_STATICCOUNT_LSB_PTR));
    #else
        return (CY_GET_REG16(TestCounter_STATICCOUNT_LSB_PTR));
    #endif /* (TestCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: TestCounter_WritePeriod
********************************************************************************
* Summary:
* Changes the period of the counter.  The new period 
* will be loaded the next time terminal count is detected.
*
* Parameters:  
*  period: (uint16) A value of 0 will result in
*         the counter remaining at zero.  
*
* Return: 
*  void
*
*******************************************************************************/
void TestCounter_WritePeriod(uint16 period) 
{
    #if(TestCounter_UsingFixedFunction)
        CY_SET_REG16(TestCounter_PERIOD_LSB_PTR,(uint16)period);
    #else
        CY_SET_REG16(TestCounter_PERIOD_LSB_PTR, period);
    #endif /* (TestCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: TestCounter_ReadPeriod
********************************************************************************
* Summary:
* Reads the current period value without affecting counter operation.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint16) Present period value.
*
*******************************************************************************/
uint16 TestCounter_ReadPeriod(void) 
{
    #if(TestCounter_UsingFixedFunction)
        return ((uint16)CY_GET_REG16(TestCounter_PERIOD_LSB_PTR));
    #else
        return (CY_GET_REG16(TestCounter_PERIOD_LSB_PTR));
    #endif /* (TestCounter_UsingFixedFunction) */
}


#if (!TestCounter_UsingFixedFunction)
/*******************************************************************************
* Function Name: TestCounter_WriteCompare
********************************************************************************
* Summary:
* Changes the compare value.  The compare output will 
* reflect the new value on the next UDB clock.  The compare output will be 
* driven high when the present counter value compares true based on the 
* configured compare mode setting. 
*
* Parameters:  
*  Compare:  New compare value. 
*
* Return: 
*  void
*
*******************************************************************************/
void TestCounter_WriteCompare(uint16 compare) \
                                   
{
    #if(TestCounter_UsingFixedFunction)
        CY_SET_REG16(TestCounter_COMPARE_LSB_PTR, (uint16)compare);
    #else
        CY_SET_REG16(TestCounter_COMPARE_LSB_PTR, compare);
    #endif /* (TestCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: TestCounter_ReadCompare
********************************************************************************
* Summary:
* Returns the compare value.
*
* Parameters:  
*  void:
*
* Return: 
*  (uint16) Present compare value.
*
*******************************************************************************/
uint16 TestCounter_ReadCompare(void) 
{
    return (CY_GET_REG16(TestCounter_COMPARE_LSB_PTR));
}


#if (TestCounter_COMPARE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: TestCounter_SetCompareMode
********************************************************************************
* Summary:
*  Sets the software controlled Compare Mode.
*
* Parameters:
*  compareMode:  Compare Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void TestCounter_SetCompareMode(uint8 compareMode) 
{
    /* Clear the compare mode bits in the control register */
    TestCounter_CONTROL &= ((uint8)(~TestCounter_CTRL_CMPMODE_MASK));
    
    /* Write the new setting */
    TestCounter_CONTROL |= compareMode;
}
#endif  /* (TestCounter_COMPARE_MODE_SOFTWARE) */


#if (TestCounter_CAPTURE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: TestCounter_SetCaptureMode
********************************************************************************
* Summary:
*  Sets the software controlled Capture Mode.
*
* Parameters:
*  captureMode:  Capture Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void TestCounter_SetCaptureMode(uint8 captureMode) 
{
    /* Clear the capture mode bits in the control register */
    TestCounter_CONTROL &= ((uint8)(~TestCounter_CTRL_CAPMODE_MASK));
    
    /* Write the new setting */
    TestCounter_CONTROL |= ((uint8)((uint8)captureMode << TestCounter_CTRL_CAPMODE0_SHIFT));
}
#endif  /* (TestCounter_CAPTURE_MODE_SOFTWARE) */


/*******************************************************************************
* Function Name: TestCounter_ClearFIFO
********************************************************************************
* Summary:
*   This function clears all capture data from the capture FIFO
*
* Parameters:  
*  void:
*
* Return: 
*  None
*
*******************************************************************************/
void TestCounter_ClearFIFO(void) 
{

    while(0u != (TestCounter_ReadStatusRegister() & TestCounter_STATUS_FIFONEMP))
    {
        (void)TestCounter_ReadCapture();
    }

}
#endif  /* (!TestCounter_UsingFixedFunction) */


/* [] END OF FILE */

