/*******************************************************************************
* File Name: TestCounter_PM.c  
* Version 3.0
*
*  Description:
*    This file provides the power management source code to API for the
*    Counter.  
*
*   Note:
*     None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "TestCounter.h"

static TestCounter_backupStruct TestCounter_backup;


/*******************************************************************************
* Function Name: TestCounter_SaveConfig
********************************************************************************
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  TestCounter_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
*******************************************************************************/
void TestCounter_SaveConfig(void) 
{
    #if (!TestCounter_UsingFixedFunction)

        TestCounter_backup.CounterUdb = TestCounter_ReadCounter();

        #if(!TestCounter_ControlRegRemoved)
            TestCounter_backup.CounterControlRegister = TestCounter_ReadControlRegister();
        #endif /* (!TestCounter_ControlRegRemoved) */

    #endif /* (!TestCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: TestCounter_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  TestCounter_backup:  Variables of this global structure are used to 
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void TestCounter_RestoreConfig(void) 
{      
    #if (!TestCounter_UsingFixedFunction)

       TestCounter_WriteCounter(TestCounter_backup.CounterUdb);

        #if(!TestCounter_ControlRegRemoved)
            TestCounter_WriteControlRegister(TestCounter_backup.CounterControlRegister);
        #endif /* (!TestCounter_ControlRegRemoved) */

    #endif /* (!TestCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: TestCounter_Sleep
********************************************************************************
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  TestCounter_backup.enableState:  Is modified depending on the enable 
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void TestCounter_Sleep(void) 
{
    #if(!TestCounter_ControlRegRemoved)
        /* Save Counter's enable state */
        if(TestCounter_CTRL_ENABLE == (TestCounter_CONTROL & TestCounter_CTRL_ENABLE))
        {
            /* Counter is enabled */
            TestCounter_backup.CounterEnableState = 1u;
        }
        else
        {
            /* Counter is disabled */
            TestCounter_backup.CounterEnableState = 0u;
        }
    #else
        TestCounter_backup.CounterEnableState = 1u;
        if(TestCounter_backup.CounterEnableState != 0u)
        {
            TestCounter_backup.CounterEnableState = 0u;
        }
    #endif /* (!TestCounter_ControlRegRemoved) */
    
    TestCounter_Stop();
    TestCounter_SaveConfig();
}


/*******************************************************************************
* Function Name: TestCounter_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  TestCounter_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void TestCounter_Wakeup(void) 
{
    TestCounter_RestoreConfig();
    #if(!TestCounter_ControlRegRemoved)
        if(TestCounter_backup.CounterEnableState == 1u)
        {
            /* Enable Counter's operation */
            TestCounter_Enable();
        } /* Do nothing if Counter was disabled before */    
    #endif /* (!TestCounter_ControlRegRemoved) */
    
}


/* [] END OF FILE */
