/*******************************************************************************
* File Name: TestTimer_PM.c
* Version 2.70
*
*  Description:
*     This file provides the power management source code to API for the
*     Timer.
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "TestTimer.h"

static TestTimer_backupStruct TestTimer_backup;


/*******************************************************************************
* Function Name: TestTimer_SaveConfig
********************************************************************************
*
* Summary:
*     Save the current user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  TestTimer_backup:  Variables of this global structure are modified to
*  store the values of non retention configuration registers when Sleep() API is
*  called.
*
*******************************************************************************/
void TestTimer_SaveConfig(void) 
{
    #if (!TestTimer_UsingFixedFunction)
        TestTimer_backup.TimerUdb = TestTimer_ReadCounter();
        TestTimer_backup.InterruptMaskValue = TestTimer_STATUS_MASK;
        #if (TestTimer_UsingHWCaptureCounter)
            TestTimer_backup.TimerCaptureCounter = TestTimer_ReadCaptureCount();
        #endif /* Back Up capture counter register  */

        #if(!TestTimer_UDB_CONTROL_REG_REMOVED)
            TestTimer_backup.TimerControlRegister = TestTimer_ReadControlRegister();
        #endif /* Backup the enable state of the Timer component */
    #endif /* Backup non retention registers in UDB implementation. All fixed function registers are retention */
}


/*******************************************************************************
* Function Name: TestTimer_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  TestTimer_backup:  Variables of this global structure are used to
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void TestTimer_RestoreConfig(void) 
{   
    #if (!TestTimer_UsingFixedFunction)

        TestTimer_WriteCounter(TestTimer_backup.TimerUdb);
        TestTimer_STATUS_MASK =TestTimer_backup.InterruptMaskValue;
        #if (TestTimer_UsingHWCaptureCounter)
            TestTimer_SetCaptureCount(TestTimer_backup.TimerCaptureCounter);
        #endif /* Restore Capture counter register*/

        #if(!TestTimer_UDB_CONTROL_REG_REMOVED)
            TestTimer_WriteControlRegister(TestTimer_backup.TimerControlRegister);
        #endif /* Restore the enable state of the Timer component */
    #endif /* Restore non retention registers in the UDB implementation only */
}


/*******************************************************************************
* Function Name: TestTimer_Sleep
********************************************************************************
*
* Summary:
*     Stop and Save the user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  TestTimer_backup.TimerEnableState:  Is modified depending on the
*  enable state of the block before entering sleep mode.
*
*******************************************************************************/
void TestTimer_Sleep(void) 
{
    #if(!TestTimer_UDB_CONTROL_REG_REMOVED)
        /* Save Counter's enable state */
        if(TestTimer_CTRL_ENABLE == (TestTimer_CONTROL & TestTimer_CTRL_ENABLE))
        {
            /* Timer is enabled */
            TestTimer_backup.TimerEnableState = 1u;
        }
        else
        {
            /* Timer is disabled */
            TestTimer_backup.TimerEnableState = 0u;
        }
    #endif /* Back up enable state from the Timer control register */
    TestTimer_Stop();
    TestTimer_SaveConfig();
}


/*******************************************************************************
* Function Name: TestTimer_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  TestTimer_backup.enableState:  Is used to restore the enable state of
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void TestTimer_Wakeup(void) 
{
    TestTimer_RestoreConfig();
    #if(!TestTimer_UDB_CONTROL_REG_REMOVED)
        if(TestTimer_backup.TimerEnableState == 1u)
        {     /* Enable Timer's operation */
                TestTimer_Enable();
        } /* Do nothing if Timer was disabled before */
    #endif /* Remove this code section if Control register is removed */
}


/* [] END OF FILE */
