/*******************************************************************************
* File Name: TimeoutTimer_PM.c
* Version 2.70
*
*  Description:
*     This file provides the power management source code to API for the
*     Timer.
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "TimeoutTimer.h"

static TimeoutTimer_backupStruct TimeoutTimer_backup;


/*******************************************************************************
* Function Name: TimeoutTimer_SaveConfig
********************************************************************************
*
* Summary:
*     Save the current user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  TimeoutTimer_backup:  Variables of this global structure are modified to
*  store the values of non retention configuration registers when Sleep() API is
*  called.
*
*******************************************************************************/
void TimeoutTimer_SaveConfig(void) 
{
    #if (!TimeoutTimer_UsingFixedFunction)
        TimeoutTimer_backup.TimerUdb = TimeoutTimer_ReadCounter();
        TimeoutTimer_backup.InterruptMaskValue = TimeoutTimer_STATUS_MASK;
        #if (TimeoutTimer_UsingHWCaptureCounter)
            TimeoutTimer_backup.TimerCaptureCounter = TimeoutTimer_ReadCaptureCount();
        #endif /* Back Up capture counter register  */

        #if(!TimeoutTimer_UDB_CONTROL_REG_REMOVED)
            TimeoutTimer_backup.TimerControlRegister = TimeoutTimer_ReadControlRegister();
        #endif /* Backup the enable state of the Timer component */
    #endif /* Backup non retention registers in UDB implementation. All fixed function registers are retention */
}


/*******************************************************************************
* Function Name: TimeoutTimer_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  TimeoutTimer_backup:  Variables of this global structure are used to
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void TimeoutTimer_RestoreConfig(void) 
{   
    #if (!TimeoutTimer_UsingFixedFunction)

        TimeoutTimer_WriteCounter(TimeoutTimer_backup.TimerUdb);
        TimeoutTimer_STATUS_MASK =TimeoutTimer_backup.InterruptMaskValue;
        #if (TimeoutTimer_UsingHWCaptureCounter)
            TimeoutTimer_SetCaptureCount(TimeoutTimer_backup.TimerCaptureCounter);
        #endif /* Restore Capture counter register*/

        #if(!TimeoutTimer_UDB_CONTROL_REG_REMOVED)
            TimeoutTimer_WriteControlRegister(TimeoutTimer_backup.TimerControlRegister);
        #endif /* Restore the enable state of the Timer component */
    #endif /* Restore non retention registers in the UDB implementation only */
}


/*******************************************************************************
* Function Name: TimeoutTimer_Sleep
********************************************************************************
*
* Summary:
*     Stop and Save the user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  TimeoutTimer_backup.TimerEnableState:  Is modified depending on the
*  enable state of the block before entering sleep mode.
*
*******************************************************************************/
void TimeoutTimer_Sleep(void) 
{
    #if(!TimeoutTimer_UDB_CONTROL_REG_REMOVED)
        /* Save Counter's enable state */
        if(TimeoutTimer_CTRL_ENABLE == (TimeoutTimer_CONTROL & TimeoutTimer_CTRL_ENABLE))
        {
            /* Timer is enabled */
            TimeoutTimer_backup.TimerEnableState = 1u;
        }
        else
        {
            /* Timer is disabled */
            TimeoutTimer_backup.TimerEnableState = 0u;
        }
    #endif /* Back up enable state from the Timer control register */
    TimeoutTimer_Stop();
    TimeoutTimer_SaveConfig();
}


/*******************************************************************************
* Function Name: TimeoutTimer_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  TimeoutTimer_backup.enableState:  Is used to restore the enable state of
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void TimeoutTimer_Wakeup(void) 
{
    TimeoutTimer_RestoreConfig();
    #if(!TimeoutTimer_UDB_CONTROL_REG_REMOVED)
        if(TimeoutTimer_backup.TimerEnableState == 1u)
        {     /* Enable Timer's operation */
                TimeoutTimer_Enable();
        } /* Do nothing if Timer was disabled before */
    #endif /* Remove this code section if Control register is removed */
}


/* [] END OF FILE */
